{-# LANGUAGE EmptyDataDecls           #-}
{-# LANGUAGE ForeignFunctionInterface #-}

-- |
-- Module:      Database.Ocilib.Enums
-- Copyright:   (c) 2016 Thierry Bourrillon
--              (c) 2016 FPInsight, Eurl.
-- License:     BSD3
-- Maintainer:  Thierry Bourrillon <thierry.bourrillon@fpinsight.com>
-- Stability:   experimental
-- Portability: portable
--
--

module Database.Ocilib.Enums where

import Database.Ocilib.BitMask


#include <ocilib.h>

{#enum define ImportMode { OCI_IMPORT_MODE_LINKAGE as OCI_IMPORT_MODE_LINKAGE
                         , OCI_IMPORT_MODE_RUNTIME as OCI_IMPORT_MODE_RUNTIME
                         } deriving (Eq, Ord, Show) #}

{#enum define CharsetMode { OCI_CHAR_ANSI as OCI_CHAR_ANSI
                          , OCI_CHAR_WIDE as OCI_CHAR_WIDE
                          } deriving (Eq, Ord, Show) #}

-- OCILIB Error types

{#enum define ErrorType { OCI_ERR_ORACLE  as OCI_ERR_ORACLE
                        , OCI_ERR_OCILIB  as OCI_ERR_OCILIB
                        , OCI_ERR_WARNING as OCI_ERR_WARNING
                        } deriving (Eq, Ord, Show) #}

-- OCILIB Error codes

{#enum define ErrorCode { OCI_ERR_NONE                   as OCI_ERR_NONE
                        , OCI_ERR_NOT_INITIALIZED        as OCI_ERR_NOT_INITIALIZED
                        , OCI_ERR_LOADING_SHARED_LIB     as OCI_ERR_LOADING_SHARED_LIB
                        , OCI_ERR_LOADING_SYMBOLS        as OCI_ERR_LOADING_SYMBOLS
                        , OCI_ERR_MULTITHREADED          as OCI_ERR_MULTITHREADED
                        , OCI_ERR_MEMORY                 as OCI_ERR_MEMORY
                        , OCI_ERR_NOT_AVAILABLE          as OCI_ERR_NOT_AVAILABLE
                        , OCI_ERR_NULL_POINTER           as OCI_ERR_NULL_POINTER
                        , OCI_ERR_DATATYPE_NOT_SUPPORTED as OCI_ERR_DATATYPE_NOT_SUPPORTED
                        , OCI_ERR_PARSE_TOKEN            as OCI_ERR_PARSE_TOKEN
                        , OCI_ERR_MAP_ARGUMENT           as OCI_ERR_MAP_ARGUMENT
                        , OCI_ERR_OUT_OF_BOUNDS          as OCI_ERR_OUT_OF_BOUNDS
                        , OCI_ERR_UNFREED_DATA           as OCI_ERR_UNFREED_DATA
                        , OCI_ERR_MAX_BIND               as OCI_ERR_MAX_BIND
                        , OCI_ERR_ATTR_NOT_FOUND         as OCI_ERR_ATTR_NOT_FOUND
                        , OCI_ERR_MIN_VALUE              as OCI_ERR_MIN_VALUE
                        , OCI_ERR_NOT_COMPATIBLE         as OCI_ERR_NOT_COMPATIBLE
                        , OCI_ERR_STMT_STATE             as OCI_ERR_STMT_STATE
                        , OCI_ERR_STMT_NOT_SCROLLABLE    as OCI_ERR_STMT_NOT_SCROLLABLE
                        , OCI_ERR_BIND_ALREADY_USED      as OCI_ERR_BIND_ALREADY_USED
                        , OCI_ERR_BIND_ARRAY_SIZE        as OCI_ERR_BIND_ARRAY_SIZE
                        , OCI_ERR_COLUMN_NOT_FOUND       as OCI_ERR_COLUMN_NOT_FOUND
                        , OCI_ERR_DIRPATH_STATE          as OCI_ERR_DIRPATH_STATE
                        , OCI_ERR_CREATE_OCI_ENVIRONMENT as OCI_ERR_CREATE_OCI_ENVIRONMENT
                        , OCI_ERR_REBIND_BAD_DATATYPE    as OCI_ERR_REBIND_BAD_DATATYPE
                        , OCI_ERR_TYPEINFO_DATATYPE      as OCI_ERR_TYPEINFO_DATATYPE
                        , OCI_ERR_ITEM_NOT_FOUND         as OCI_ERR_ITEM_NOT_FOUND
                        , OCI_ERR_ARG_INVALID_VALUE      as OCI_ERR_ARG_INVALID_VALUE
                        } deriving (Eq, Ord, Show) #}

-- allocated bytes types --

{#enum define AllocatedBytesType { OCI_MEM_ORACLE as OCI_MEM_ORACLE
                                 , OCI_MEM_OCILIB as OCI_MEM_OCILIB
                                 , OCI_MEM_ALL    as OCI_MEM_ALL
                                 } deriving (Eq, Ord, Show) #}

-- binding --

{#enum define BindMode { OCI_BIND_BY_POS as OCI_BIND_BY_POS
                       , OCI_BIND_BY_NAME as OCI_BIND_BY_NAME
                       } deriving (Eq, Ord, Show) #}


-- fetching --
{#enum define LongMode { OCI_LONG_EXPLICIT as OCI_LONG_EXPLICIT
                       , OCI_LONG_IMPLICIT as OCI_LONG_IMPLICIT
                       } deriving (Eq, Ord, Show) #}

{-
-- unknown value --

OCI_UNKNOWN                         0
-}

-- C Data Type mapping --
{#enum define ColumnType { OCI_CDT_NUMERIC    as OCI_CDT_NUMERIC
                         , OCI_CDT_DATETIME   as OCI_CDT_DATETIME
                         , OCI_CDT_TEXT       as OCI_CDT_TEXT
                         , OCI_CDT_LONG       as OCI_CDT_LONG
                         , OCI_CDT_CURSOR     as OCI_CDT_CURSOR
                         , OCI_CDT_LOB        as OCI_CDT_LOB
                         , OCI_CDT_FILE       as OCI_CDT_FILE
                         , OCI_CDT_TIMESTAMP  as OCI_CDT_TIMESTAMP
                         , OCI_CDT_INTERVAL   as OCI_CDT_INTERVAL
                         , OCI_CDT_RAW        as OCI_CDT_RAW
                         , OCI_CDT_OBJECT     as OCI_CDT_OBJECT
                         , OCI_CDT_COLLECTION as OCI_CDT_COLLECTION
                         , OCI_CDT_REF        as OCI_CDT_REF
                         , OCI_CDT_BOOLEAN    as OCI_CDT_BOOLEAN
                         } deriving (Eq, Ord, Show) #}

-- Data Type codes for OCI_ImmediateXXX() calls --
{#enum define DataTypeCode { OCI_ARG_SHORT      as OCI_ARG_SHORT
                           , OCI_ARG_USHORT     as OCI_ARG_USHORT
                           , OCI_ARG_INT        as OCI_ARG_INT
                           , OCI_ARG_UINT       as OCI_ARG_UINT
                           , OCI_ARG_BIGINT     as OCI_ARG_BIGINT
                           , OCI_ARG_BIGUINT    as OCI_ARG_BIGUINT
                           , OCI_ARG_DOUBLE     as OCI_ARG_DOUBLE
                           , OCI_ARG_DATETIME   as OCI_ARG_DATETIME
                           , OCI_ARG_TEXT       as OCI_ARG_TEXT
                           , OCI_ARG_LOB        as OCI_ARG_LOB
                           , OCI_ARG_FILE       as OCI_ARG_FILE
                           , OCI_ARG_TIMESTAMP  as OCI_ARG_TIMESTAMP
                           , OCI_ARG_INTERVAL   as OCI_ARG_INTERVAL
                           , OCI_ARG_RAW        as OCI_ARG_RAW
                           , OCI_ARG_OBJECT     as OCI_ARG_OBJECT
                           , OCI_ARG_COLLECTION as OCI_ARG_COLLECTION
                           , OCI_ARG_REF        as OCI_ARG_REF
                           , OCI_ARG_FLOAT      as OCI_ARG_FLOAT
                           } deriving (Eq, Ord, Show) #}


-- statement types --

{#enum define StatementType { OCI_CST_SELECT as OCI_CST_SELECT
                            , OCI_CST_UPDATE as OCI_CST_UPDATE
                            , OCI_CST_DELETE as OCI_CST_DELETE
                            , OCI_CST_INSERT as OCI_CST_INSERT
                            , OCI_CST_CREATE as OCI_CST_CREATE
                            , OCI_CST_DROP as OCI_CST_DROP
                            , OCI_CST_ALTER as OCI_CST_ALTER
                            , OCI_CST_BEGIN as OCI_CST_BEGIN
                            , OCI_CST_DECLARE as OCI_CST_DECLARE
                            , OCI_CST_CALL as OCI_CST_CALL
                            } deriving (Eq, Ord, Show) #}

-- environment modes --

{#enum define EnvironmentMode { OCI_ENV_DEFAULT as OCI_ENV_DEFAULT
                              , OCI_ENV_THREADED as OCI_ENV_THREADED
                              , OCI_ENV_CONTEXT as OCI_ENV_CONTEXT
                              , OCI_ENV_EVENTS as OCI_ENV_EVENTS
                              } deriving (Eq, Ord, Show) #}

instance ToBitMask EnvironmentMode

-- sessions modes --

{#enum define SessionMode { OCI_SESSION_DEFAULT as OCI_SESSION_DEFAULT
                          , OCI_SESSION_XA as OCI_SESSION_XA
                          , OCI_SESSION_SYSDBA as OCI_SESSION_SYSDBA
                          , OCI_SESSION_SYSOPER as OCI_SESSION_SYSOPER
                          , OCI_SESSION_PRELIM_AUTH as OCI_SESSION_PRELIM_AUTH
                          } deriving (Eq, Ord, Show) #}


-- change notification types --

{#enum define ChangeNotificationType { OCI_CNT_OBJECTS   as OCI_CNT_OBJECTS
                                     , OCI_CNT_ROWS      as OCI_CNT_ROWS
                                     , OCI_CNT_DATABASES as OCI_CNT_DATABASES
                                     , OCI_CNT_ALL       as OCI_CNT_ALL
                                     } deriving (Eq, Ord, Show) #}

-- event notification types --

{#enum define EventNotificationType { OCI_ENT_STARTUP        as OCI_ENT_STARTUP
                                    , OCI_ENT_SHUTDOWN       as OCI_ENT_SHUTDOWN
                                    , OCI_ENT_SHUTDOWN_ANY   as OCI_ENT_SHUTDOWN_ANY
                                    , OCI_ENT_DROP_DATABASE  as OCI_ENT_DROP_DATABASE
                                    , OCI_ENT_DEREGISTER     as OCI_ENT_DEREGISTER
                                    , OCI_ENT_OBJECT_CHANGED as OCI_ENT_OBJECT_CHANGED
                                    } deriving (Eq, Ord, Show) #}

-- event object notification types --

{#enum define EventObjectNotificationType { OCI_ONT_INSERT  as OCI_ONT_INSERT
                                          , OCI_ONT_UPDATE  as OCI_ONT_UPDATE
                                          , OCI_ONT_DELETE  as OCI_ONT_DELETE
                                          , OCI_ONT_ALTER   as OCI_ONT_ALTER
                                          , OCI_ONT_DROP    as OCI_ONT_DROP
                                          , OCI_ONT_GENERIC as OCI_ONT_GENERIC
                                          } deriving (Eq, Ord, Show) #}

-- database startup modes --

{#enum define DatabaseStartupMode { OCI_DB_SPM_START as OCI_DB_SPM_START
                                  , OCI_DB_SPM_MOUNT as OCI_DB_SPM_MOUNT
                                  , OCI_DB_SPM_OPEN  as OCI_DB_SPM_OPEN
                                  , OCI_DB_SPM_FULL  as OCI_DB_SPM_FULL
                                  } deriving (Eq, Ord, Show) #}

-- database startup flags --

{#enum define DatabaseStartupFlag { OCI_DB_SPF_DEFAULT  as OCI_DB_SPF_DEFAULT
                                  , OCI_DB_SPF_FORCE    as OCI_DB_SPF_FORCE
                                  , OCI_DB_SPF_RESTRICT as OCI_DB_SPF_RESTRICT
                                  } deriving (Eq, Ord, Show) #}


-- database shutdown modes --

{#enum define DatabaseShutdownMode { OCI_DB_SDM_SHUTDOWN as OCI_DB_SDM_SHUTDOWN
                                   , OCI_DB_SDM_CLOSE    as OCI_DB_SDM_CLOSE
                                   , OCI_DB_SDM_DISMOUNT as OCI_DB_SDM_DISMOUNT
                                   , OCI_DB_SDM_FULL     as OCI_DB_SDM_FULL
                                   } deriving (Eq, Ord, Show) #}

-- database shutdown flags --

{#enum define DatabaseShutdownFlag { OCI_DB_SDF_DEFAULT     as OCI_DB_SDF_DEFAULT
                                   , OCI_DB_SDF_TRANS       as OCI_DB_SDF_TRANS
                                   , OCI_DB_SDF_TRANS_LOCAL as OCI_DB_SDF_TRANS_LOCAL
                                   , OCI_DB_SDF_IMMEDIATE   as OCI_DB_SDF_IMMEDIATE
                                   , OCI_DB_SDF_ABORT       as OCI_DB_SDF_ABORT
                                   } deriving (Eq, Ord, Show) #}

-- charset form types --
{#enum define CharsetFormType { OCI_CSF_NONE     as OCI_CSF_NONE
                              , OCI_CSF_DEFAULT  as OCI_CSF_DEFAULT
                              , OCI_CSF_NATIONAL as OCI_CSF_NATIONAL
                              } deriving (Eq, Ord, Show) #}

-- statement fetch mode --

{#enum define StatementFetchMode { OCI_SFM_DEFAULT    as OCI_SFM_DEFAULT
                                 , OCI_SFM_SCROLLABLE as OCI_SFM_SCROLLABLE
                                 } deriving (Eq, Ord, Show) #}

-- statement fetch direction --

{#enum define StatementFetchDirection { OCI_SFD_ABSOLUTE as OCI_SFD_ABSOLUTE
                                      , OCI_SFD_RELATIVE as OCI_SFD_RELATIVE
                                      } deriving (Eq, Ord, Show) #}

-- bind allocation mode --

{#enum define BindAllocationMode { OCI_BAM_EXTERNAL as OCI_BAM_EXTERNAL
                                 , OCI_BAM_INTERNAL as OCI_BAM_INTERNAL
                                 } deriving (Eq, Ord, Show) #}

-- bind direction mode --

{#enum define BindDirectionMode { OCI_BDM_IN     as OCI_BDM_IN
                                , OCI_BDM_OUT    as OCI_BDM_OUT
                                , OCI_BDM_IN_OUT as OCI_BDM_IN_OUT
                                } deriving (Eq, Ord, Show) #}

-- Column property flags --

{#enum define ColumnPropertyFlag { OCI_CPF_NONE                      as OCI_CPF_NONE
                                 , OCI_CPF_IS_IDENTITY               as OCI_CPF_IS_IDENTITY
                                 , OCI_CPF_IS_GEN_ALWAYS             as OCI_CPF_IS_GEN_ALWAYS
                                 , OCI_CPF_IS_GEN_BY_DEFAULT_ON_NULL as OCI_CPF_IS_GEN_BY_DEFAULT_ON_NULL
                                 } deriving (Eq, Ord, Show) #}

{-
-- Integer sign flag --

OCI_NUM_UNSIGNED                    2

-- External Integer types --

OCI_NUM_SHORT                       4
OCI_NUM_INT                         8
OCI_NUM_BIGINT                      16
OCI_NUM_FLOAT                       32
OCI_NUM_DOUBLE                      64

OCI_NUM_USHORT                      (OCI_NUM_SHORT  | OCI_NUM_UNSIGNED)
OCI_NUM_UINT                        (OCI_NUM_INT    | OCI_NUM_UNSIGNED)
OCI_NUM_BIGUINT                     (OCI_NUM_BIGINT | OCI_NUM_UNSIGNED)

-}

-- timestamp types --

{#enum define TimestampType { OCI_TIMESTAMP     as OCI_TIMESTAMP
                            , OCI_TIMESTAMP_TZ  as OCI_TIMESTAMP_TZ
                            , OCI_TIMESTAMP_LTZ as OCI_TIMESTAMP_LTZ
                            } deriving (Eq, Ord, Show) #}

-- interval types --
{#enum define IntervalType { OCI_INTERVAL_YM as OCI_INTERVAL_YM
                           , OCI_INTERVAL_DS as OCI_INTERVAL_DS
                           } deriving (Eq, Ord, Show) #}

-- long types --

{#enum define LongType { OCI_BLONG as OCI_BLONG
                       , OCI_CLONG as OCI_CLONG
                       } deriving (Eq, Ord, Show) #}

-- lob types --

{#enum define LobType { OCI_BLOB as OCI_BLOB
                      , OCI_CLOB as OCI_CLOB
                      , OCI_NCLOB as OCI_NCLOB
                      } deriving (Eq, Ord, Show) #}

-- lob opening mode --

{#enum define LobOpeningMode { OCI_LOB_READONLY as OCI_LOB_READONLY
                             , OCI_LOB_READWRITE as OCI_LOB_READWRITE
                             } deriving (Eq, Ord, Show) #}

-- file types --

{#enum define FileType { OCI_BFILE as OCI_BFILE
                       , OCI_CFILE as OCI_CFILE
                       } deriving (Eq, Ord, Show) #}

-- lob browsing mode --

{#enum define LobBrowsingMode { OCI_SEEK_SET as OCI_SEEK_SET
                              , OCI_SEEK_END as OCI_SEEK_END
                              , OCI_SEEK_CUR as OCI_SEEK_CUR
                              } deriving (Eq, Ord, Show) #}

-- type info types --

{#enum define TypeInfoType { OCI_TIF_TABLE as OCI_TIF_TABLE
                           , OCI_TIF_VIEW  as OCI_TIF_VIEW
                           , OCI_TIF_TYPE  as OCI_TIF_TYPE
                           } deriving (Eq, Ord, Show) #}

-- object type --

{#enum define ObjectType { OCI_OBJ_PERSISTENT as OCI_OBJ_PERSISTENT
                         , OCI_OBJ_TRANSIENT  as OCI_OBJ_TRANSIENT
                         , OCI_OBJ_VALUE      as OCI_OBJ_VALUE
                         } deriving (Eq, Ord, Show) #}

-- collection types --
{#enum define CollectionType { OCI_COLL_VARRAY        as OCI_COLL_VARRAY
                             , OCI_COLL_NESTED_TABLE  as OCI_COLL_NESTED_TABLE
                             , OCI_COLL_INDEXED_TABLE as OCI_COLL_INDEXED_TABLE
                             } deriving (Eq, Ord, Show) #}

-- pool types --
{#enum define PoolType { OCI_POOL_CONNECTION as OCI_POOL_CONNECTION
                       , OCI_POOL_SESSION as OCI_POOL_SESSION
                       } deriving (Eq, Ord, Show) #}

-- AQ message state --
{#enum define AQMessageState { OCI_AMS_READY as OCI_AMS_READY
                             , OCI_AMS_WAITING as OCI_AMS_WAITING
                             , OCI_AMS_PROCESSED as OCI_AMS_PROCESSED
                             , OCI_AMS_EXPIRED as OCI_AMS_EXPIRED
                             } deriving (Eq, Ord, Show) #}

-- AQ sequence deviation --
{#enum define AQSequenceDeviation { OCI_ASD_BEFORE as OCI_ASD_BEFORE
                                  , OCI_ASD_TOP as OCI_ASD_TOP
                                  } deriving (Eq, Ord, Show) #}

{-
-- AQ message visibility --

OCI_AMV_IMMEDIATE                   1
OCI_AMV_ON_COMMIT                   2

-- AQ dequeue mode --

OCI_ADM_BROWSE                      1
OCI_ADM_LOCKED                      2
OCI_ADM_REMOVE                      3
OCI_ADM_REMOVE_NODATA               4

-- AQ dequeue navigation --

OCI_ADN_FIRST_MSG                   1
OCI_ADN_NEXT_TRANSACTION            2
OCI_ADN_NEXT_MSG                    3

-- AQ queue table purge mode --

OCI_APM_BUFFERED                    1
OCI_APM_PERSISTENT                  2
OCI_APM_ALL                         (OCI_APM_BUFFERED | OCI_APM_PERSISTENT)

-- AQ queue table grouping mode --

OCI_AGM_NONE                        0
OCI_AGM_TRANSACTIONNAL              1

-- AQ queue table type --

OCI_AQT_NORMAL                      0
OCI_AQT_EXCEPTION                   1
OCI_AQT_NON_PERSISTENT              2


-- direct path processing return status --

OCI_DPR_COMPLETE                    1
OCI_DPR_ERROR                       2
OCI_DPR_FULL                        3
OCI_DPR_PARTIAL                     4
OCI_DPR_EMPTY                       5

-- direct path conversion modes --

OCI_DCM_DEFAULT                     1
OCI_DCM_FORCE                       2

-- trace size constants --

OCI_SIZE_TRACE_ID                   64
OCI_SIZE_TRACE_MODULE               48
OCI_SIZE_TRACE_ACTION               32
OCI_SIZE_TRACE_INFO                 64

-- trace types --

OCI_TRC_IDENTITY                    1
OCI_TRC_MODULE                      2
OCI_TRC_ACTION                      3
OCI_TRC_DETAIL                      4

-- HA event type --

OCI_HET_DOWN                        0
OCI_HET_UP                          1

-- HA event source --
OCI_HES_INSTANCE                    0
OCI_HES_DATABASE                    1
OCI_HES_NODE                        2
OCI_HES_SERVICE                     3
OCI_HES_SERVICE_MEMBER              4
OCI_HES_ASM_INSTANCE                5
OCI_HES_PRECONNECT                  6

-- Fail over types --

OCI_FOT_NONE                        1
OCI_FOT_SESSION                     2
OCI_FOT_SELECT                      4

-- fail over notifications --

OCI_FOE_END                         1
OCI_FOE_ABORT                       2
OCI_FOE_REAUTH                      4
OCI_FOE_BEGIN                       8
OCI_FOE_ERROR                       16

-- fail over callback return code --

OCI_FOC_OK                          0
OCI_FOC_RETRY                       25410

-- hash tables support --

OCI_HASH_STRING                     1
OCI_HASH_INTEGER                    2
OCI_HASH_POINTER                    3

-- transaction types --

OCI_TRS_NEW                         0x00000001
OCI_TRS_READONLY                    0x00000100
OCI_TRS_READWRITE                   0x00000200
OCI_TRS_SERIALIZABLE                0x00000400
OCI_TRS_LOOSE                       0x00010000
OCI_TRS_TIGHT                       0x00020000

-- format types --

OCI_FMT_DATE                        1
OCI_FMT_TIMESTAMP                   2
OCI_FMT_NUMERIC                     3
OCI_FMT_BINARY_DOUBLE               4
OCI_FMT_BINARY_FLOAT                5

-- sql function codes --
OCI_SFC_CREATE_TABLE                1
OCI_SFC_SET_ROLE                    2
OCI_SFC_INSERT                      3
OCI_SFC_SELECT                      4
OCI_SFC_UPDATE                      5
OCI_SFC_DROP_ROLE                   6
OCI_SFC_DROP_VIEW                   7
OCI_SFC_DROP_TABLE                  8
OCI_SFC_DELETE                      9
OCI_SFC_CREATE_VIEW                 10
OCI_SFC_DROP_USER                   11
OCI_SFC_CREATE_ROLE                 12
OCI_SFC_CREATE_SEQUENCE             13
OCI_SFC_ALTER_SEQUENCE              14
OCI_SFC_DROP_SEQUENCE               16
OCI_SFC_CREATE_SCHEMA               17
OCI_SFC_CREATE_CLUSTER              18
OCI_SFC_CREATE_USER                 19
OCI_SFC_CREATE_INDEX                20
OCI_SFC_DROP_INDEX                  21
OCI_SFC_DROP_CLUSTER                22
OCI_SFC_VALIDATE_INDEX              23
OCI_SFC_CREATE_PROCEDURE            24
OCI_SFC_ALTER_PROCEDURE             25
OCI_SFC_ALTER_TABLE                 26
OCI_SFC_EXPLAIN                     27
OCI_SFC_GRANT                       28
OCI_SFC_REVOKE                      29
OCI_SFC_CREATE_SYNONYM              30
OCI_SFC_DROP_SYNONYM                31
OCI_SFC_ALTER_SYSTEM_SWITCHLOG      32
OCI_SFC_SET_TRANSACTION             33
OCI_SFC_PLSQL_EXECUTE               34
OCI_SFC_LOCK                        35
OCI_SFC_NOOP                        36
OCI_SFC_RENAME                      37
OCI_SFC_COMMENT                     38
OCI_SFC_AUDIT                       39
OCI_SFC_NO_AUDIT                    40
OCI_SFC_ALTER_INDEX                 41
OCI_SFC_CREATE_EXTERNAL_DATABASE    42
OCI_SFC_DROP_EXTERNALDATABASE       43
OCI_SFC_CREATE_DATABASE             44
OCI_SFC_ALTER_DATABASE              45
OCI_SFC_CREATE_ROLLBACK_SEGMENT     46
OCI_SFC_ALTER_ROLLBACK_SEGMENT      47
OCI_SFC_DROP_ROLLBACK_SEGMENT       48
OCI_SFC_CREATE_TABLESPACE           49
OCI_SFC_ALTER_TABLESPACE            50
OCI_SFC_DROP_TABLESPACE             51
OCI_SFC_ALTER_SESSION               52
OCI_SFC_ALTER_USER                  53
OCI_SFC_COMMIT_WORK                 54
OCI_SFC_ROLLBACK                    55
OCI_SFC_SAVEPOINT                   56
OCI_SFC_CREATE_CONTROL_FILE         57
OCI_SFC_ALTER_TRACING               58
OCI_SFC_CREATE_TRIGGER              59
OCI_SFC_ALTER_TRIGGER               60
OCI_SFC_DROP_TRIGGER                61
OCI_SFC_ANALYZE_TABLE               62
OCI_SFC_ANALYZE_INDEX               63
OCI_SFC_ANALYZE_CLUSTER             64
OCI_SFC_CREATE_PROFILE              65
OCI_SFC_DROP_PROFILE                66
OCI_SFC_ALTER_PROFILE               67
OCI_SFC_DROP_PROCEDURE              68
OCI_SFC_ALTER_RESOURCE_COST         70
OCI_SFC_CREATE_SNAPSHOT_LOG         71
OCI_SFC_ALTER_SNAPSHOT_LOG          72
OCI_SFC_DROP_SNAPSHOT_LOG           73
OCI_SFC_DROP_SUMMARY                73
OCI_SFC_CREATE_SNAPSHOT             74
OCI_SFC_ALTER_SNAPSHOT              75
OCI_SFC_DROP_SNAPSHOT               76
OCI_SFC_CREATE_TYPE                 77
OCI_SFC_DROP_TYPE                   78
OCI_SFC_ALTER_ROLE                  79
OCI_SFC_ALTER_TYPE                  80
OCI_SFC_CREATE_TYPE_BODY            81
OCI_SFC_ALTER_TYPE_BODY             82
OCI_SFC_DROP_TYPE_BODY              83
OCI_SFC_DROP_LIBRARY                84
OCI_SFC_TRUNCATE_TABLE              85
OCI_SFC_TRUNCATE_CLUSTER            86
OCI_SFC_CREATE_BITMAPFILE           87
OCI_SFC_ALTER_VIEW                  88
OCI_SFC_DROP_BITMAPFILE             89
OCI_SFC_SET_CONSTRAINTS             90
OCI_SFC_CREATE_FUNCTION             91
OCI_SFC_ALTER_FUNCTION              92
OCI_SFC_DROP_FUNCTION               93
OCI_SFC_CREATE_PACKAGE              94
OCI_SFC_ALTER_PACKAGE               95
OCI_SFC_DROP_PACKAGE                96
OCI_SFC_CREATE_PACKAGE_BODY         97
OCI_SFC_ALTER_PACKAGE_BODY          98
OCI_SFC_DROP_PACKAGE_BODY           99
OCI_SFC_CREATE_DIRECTORY            157
OCI_SFC_DROP_DIRECTORY              158
OCI_SFC_CREATE_LIBRARY              159
OCI_SFC_CREATE_JAVA                 160
OCI_SFC_ALTER_JAVA                  161
OCI_SFC_DROP_JAVA                   162
OCI_SFC_CREATE_OPERATOR             163
OCI_SFC_CREATE_INDEXTYPE            164
OCI_SFC_DROP_INDEXTYPE              165
OCI_SFC_ALTER_INDEXTYPE             166
OCI_SFC_DROP_OPERATOR               167
OCI_SFC_ASSOCIATE_STATISTICS        168
OCI_SFC_DISASSOCIATE_STATISTICS     169
OCI_SFC_CALL_METHOD                 170
OCI_SFC_CREATE_SUMMARY              171
OCI_SFC_ALTER_SUMMARY               172
OCI_SFC_CREATE_DIMENSION            174
OCI_SFC_ALTER_DIMENSION             175
OCI_SFC_DROP_DIMENSION              176
OCI_SFC_CREATE_CONTEXT              177
OCI_SFC_DROP_CONTEXT                178
OCI_SFC_ALTER_OUTLINE               179
OCI_SFC_CREATE_OUTLINE              180
OCI_SFC_DROP_OUTLINE                181
OCI_SFC_UPDATE_INDEXES              182
OCI_SFC_ALTER_OPERATOR              183

-}
